
/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "peptidewindow.h"
#include "../project_view/projectwindow.h"
#include "ui_peptide_detail_view.h"
#include <pappsomspp/exception/exceptionnotfound.h>
#include <pappsomspp/spectrum/qualifiedspectrum.h>
#include <QMessageBox>
#include <QFileDialog>
#include <QSettings>
#include <QDebug>


void SpectrumSpLoaderThread::doLoadSpectrumSp (PeptideMatch * p_peptide_match) {
    qDebug() << "SpectrumSpLoaderThread::doLoadSpectrumSp begin";
    pappso::SpectrumSp spectrum;
    try {
        spectrum = p_peptide_match->getIdentificationDataSource()->getSpectrumSp(p_peptide_match->getScan());
        emit spectrumSpReady(spectrum, QString(""));
    }

    catch (pappso::ExceptionNotFound & error) {
        emit spectrumSpReady(spectrum, error.qwhat());
    }
    qDebug() << "SpectrumSpLoaderThread::doLoadSpectrumSp end";
}

PeptideWindow::PeptideWindow(ProjectWindow *parent):
    QMainWindow(parent),
    ui(new Ui::PeptideDetailView)
{
    qDebug() << "PeptideWindow::PeptideWindow begin";
    _p_project_window = parent;
    ui->setupUi(this);

    _p_spectrum_overlay = new LoadingOverlay(ui->spectrumWidget);
    /*
    */
    SpectrumSpLoaderThread * worker = new SpectrumSpLoaderThread;
    worker->moveToThread(&_spectrum_loader_thread);
    _spectrum_loader_thread.start();


    QSettings settings;
    QString unit = settings.value("peptideview/precision_unit", "dalton").toString();
    pappso::mz precision = settings.value("peptideview/precision_value", "0.2").toDouble();

    if (unit == "dalton") {
        _p_precision = Precision::getDaltonInstance(precision);
    } else {
        _p_precision = Precision::getPpmInstance(precision);
    }

    ui->file_not_found->setVisible(false);

    qRegisterMetaType<pappso::SpectrumSp>("pappso::SpectrumSp");
#if QT_VERSION >= 0x050000
    // Qt5 code
    /*
    connect(&workerThread, &QThread::finished, worker, &QObject::deleteLater);
    connect(this, &PtSpectrumViewer::operateMsDataFile, worker, &PwizLoaderThread::doMsDataFileLoad);
    connect(worker, &PwizLoaderThread::msDataReady, this, &PtSpectrumViewer::handleMsDataFile);
    */
#else
// Qt4 code
    connect (_p_project_window, SIGNAL(identificationGroupGrouped(IdentificationGroup *)), this,SLOT(doIdentificationGroupGrouped(IdentificationGroup *)));

    connect (this, SIGNAL(loadSpectrumSp(PeptideMatch *)), worker,SLOT(doLoadSpectrumSp(PeptideMatch *)));
    connect (worker, SIGNAL(spectrumSpReady(pappso::SpectrumSp, QString)), this,SLOT(doSpectrumSpReady(pappso::SpectrumSp, QString)));
    connect (_p_spectrum_overlay,SIGNAL(mzChanged(double)), this, SLOT(setMz(double)));
    //connect(_protein_table_model_p, SIGNAL(layoutChanged()), this, SLOT(updateStatusBar()));
#endif

    qDebug() << "PeptideWindow::PeptideWindow end";
}

PeptideWindow::~PeptideWindow()
{
    delete _p_spectrum_overlay;
    delete ui;
}


void PeptideWindow::setMz(double mz) {
}

void PeptideWindow::doIdentificationGroupGrouped(IdentificationGroup * p_identification_group) {
    updateDisplay();
}

void PeptideWindow::updateDisplay() {
    try {
        ui->sequence_label->setText(_p_peptide_match->getPeptideXtpSp().get()->toString());
        ui->mz_label->setText(QString("%1").arg(_p_peptide_match->getPeptideXtpSp().get()->getMz(_p_peptide_match->getCharge())));
        ui->z_label->setText(QString("%1").arg(_p_peptide_match->getCharge()));
        ui->scan_label->setText(QString("%1").arg(_p_peptide_match->getScan()));
        ui->modification_label->setText(_p_peptide_match->getPeptideXtpSp().get()->getModifString());


    }
    catch (pappso::PappsoException exception_pappso) {
        QMessageBox::warning(this,
                             tr("Unable to display protein details :"), exception_pappso.qwhat());
    }
    catch (std::exception exception_std) {
        QMessageBox::warning(this,
                             tr("Unable to display protein details :"), exception_std.what());
    }

}


void PeptideWindow::chooseDefaultMzDataDir() {
    QSettings settings;
    QString default_location = settings.value("path/mzdatadir", "").toString();

    QString filename = QFileDialog::getExistingDirectory(this,
                       tr("Choose default mz data directory"), default_location,
                       QFileDialog::ShowDirsOnly);

    if (filename.isEmpty()) {
        return;
    }
    QString path = QFileInfo(filename).absoluteFilePath();
    settings.setValue("path/mzdatadir", path);
    ui->mz_data_dir_label->setText(path);


    qDebug() << "PeptideWindow::chooseDefaultMzDataDir begin";
    _p_spectrum_overlay->displayLoadingMessage();
    ui->file_not_found->setVisible(false);
    ui->spectrumWidget->setVisible(true);
    emit loadSpectrumSp(_p_peptide_match);
    ui->statusbar->showMessage(tr("loading spectrum"));

}


void PeptideWindow::doSpectrumSpReady(SpectrumSp spectrum_sp, QString error) {
    qDebug() << "PeptideWindow::doSpectrumSpReady begin";
    _p_spectrum_overlay->hideLoadingMessage();
    if (!error.isEmpty()) {
        //not found

        ui->statusbar->showMessage(tr("spectrum not found"));
        QSettings settings;
        QString path = settings.value("path/mzdatadir", "").toString();

        ui->mz_data_dir_label->setText(path);

        ui->file_not_found->setVisible(true);
        ui->spectrumWidget->setVisible(false);
    }
    else {
        ui->statusbar->showMessage(tr(""));
        pappso::QualifiedSpectrum spectrum_copy;
        spectrum_copy.setPrecursorCharge(_p_peptide_match->getCharge());
        spectrum_copy.setOriginalSpectrumSp(spectrum_sp);

        ui->spectrumWidget->setQualifiedSpectrum(spectrum_copy);
        ui->file_not_found->setVisible(false);
        ui->spectrumWidget->setVisible(true);
    }
    qDebug() << "PeptideWindow::doSpectrumSpReady end";

}

void PeptideWindow::setPeptideMatch(PeptideMatch * p_peptide_match) {
    qDebug() << "PeptideWindow::setPeptideMatch begin";
    _p_peptide_match = p_peptide_match;

    qDebug() << "PeptideWindow::setPeptideMatch emit loadSpectrumSp(_p_peptide_match)";
    emit loadSpectrumSp(_p_peptide_match);

    _p_spectrum_overlay->displayLoadingMessage();
    ui->file_not_found->setVisible(false);
    ui->spectrumWidget->setVisible(true);
    ui->statusbar->showMessage(tr("loading spectrum"));

    pappso::PeptideSp peptide = _p_peptide_match->getPeptideXtpSp();
    ui->spectrumWidget->setPrecision(_p_precision);
    ui->spectrumWidget->setPeptideSp(peptide);

    updateDisplay();
    qDebug() << "PeptideWindow::setPeptideMatch end";
}
